#define CloudThickness 4 //[1 2 4 8 16]
#define CloudAmount 11.0 //[13.0 12.0 11.0 10.0 9.0]
#define CloudHeight 16.0 //[8.0 12.0 16.0 20.0 24.0]
#define CloudOpacity 1.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]

vec3 drawCloud(vec3 fragpos, float dither, vec3 color, vec3 light, vec3 ambient) {
	float totalcloud = 0;
	vec3 sVector = normalize(fragpos);
	float cosT = dot(sVector,upVec);
	float cosS = dot(sunVec,sVector);
	float pi = 3.1415927;
	vec2 wind = vec2(frametime*0.001,0.0);

	vec3 tpos = (gbufferModelViewInverse * vec4(fragpos,1.0)).xyz;
	vec3 wVector = normalize(tpos);

	float cloud = 0.0;
	float cloudalpha = 0.0;
	vec3 cloud_color = ambient;
	float colmix = 0.0;
	float colmult = (0.5-moonVisibility*0.35)*(1.0-wetness*(0.8-0.4*sunVisibility));

	if (cosT > 0.05){
		for (int i = 0; i < 8; i++) {
			vec3 intersection = wVector*((CloudHeight+(i+dither)/2.0)/wVector.y) * 0.002;
			vec2 coord = cameraPosition.xz * 0.00025 + intersection.xz;
			
			float noise = 0.0;
			if (cloud < 0.999){
				noise+= texture2D(noisetex,coord*0.5+wind*0.55).x;
				noise+= texture2D(noisetex,coord*0.25+wind*0.45).x*2;
				noise+= texture2D(noisetex,coord*0.125+wind*0.35).x*3;
				noise+= texture2D(noisetex,coord*0.0625+wind*0.25).x*4;
				noise+= texture2D(noisetex,coord*0.03125+wind*0.15).x*5;
				noise+= texture2D(noisetex,coord*0.016125+wind*0.05).x*6;
				noise = max(mix(noise,21.0,0.4*wetness)*min(sqrt(cosT*6.0),1.0)-(CloudAmount+pow(abs(float(i-4.0+dither))/2.0,2.0)),0.0)*(1.0-0.5*wetness)*sqrt(CloudThickness/4.0);
				noise = noise/pow(1.0+pow(noise,CloudThickness),1.0/CloudThickness);
				}
			float hnoise = (texture2D(noisetex,coord*0.25+wind*0.4).x + texture2D(noisetex,coord*0.5+wind*0.5).x)*0.5;
			cloud_color = mix(cloud_color,mix(ambient*((1.0-0.2*wetness)*(1.0-0.25*moonVisibility)),light,pow(colmix+0.125*(hnoise+dither),2.5)),noise*(1.0-cloud*cloud));
			cloud = max(cloud,noise);
			colmix += 0.125;
			cloudalpha += cloud;
		}
		cloudalpha *= 0.125;
		cloud_color *= (1.0+4.0*(1.0-cloudalpha*wetness)*pow(max(cosS,0.0),8.0)*wetness)*(2.0-eBS)*(1.0+nightVision);
		cloud *= clamp(cosT*10.0-0.5,0.0,1.0)*(1.0-0.7*wetness*wetness);
	}
	return mix(color,cloud_color*colmult,cloud*cloud*CloudOpacity);
}
float getnoise(vec2 pos){
	return fract(sin(dot(pos, vec2(12.9898, 4.1414))) * 43758.5453);
}

vec3 drawStars(vec3 fragpos, vec3 color, vec3 light){
	vec3 wpos = vec3(gbufferModelViewInverse * vec4(fragpos,1.0));
	vec3 intersection = wpos/(wpos.y+length(wpos.xz));
	vec2 wind = vec2(frametime,0.0);
	vec2 coord = floor((intersection.xz*0.4+cameraPosition.xz*0.0001+wind*0.00125)*1024.0)/1024.0;
	
	float NdotU = sqrt(sqrt(max(dot(normalize(fragpos),normalize(upVec)),0.0)));
	
	float star = 1.0;
	if (NdotU > 0.0){
		star *= getnoise(coord.xy);
		star *= getnoise(coord.xy+0.1);
		star *= getnoise(coord.xy+0.23);
	}
	star = max(star-0.825,0.0)*5.0*NdotU*(1.0-rainStrength)*moonVisibility;
		
	return color + star*pow(light,vec3(0.8));
}