#version 120

/*
BSL Shaders by Capt Tatsu
http://bitslablab.wixsite.com/main
*/

#define Weather
#define WeatherOpacity 1.00 //[0.25 0.50 0.75 1.00 1.25 1.50 1.75 2.00]

varying vec2 lmcoord;
varying vec2 texcoord;

varying vec3 upVec;
varying vec3 sunVec;

uniform int isEyeInWater;
uniform int worldTime;

uniform float nightVision;
uniform float rainStrength;
uniform float viewWidth;
uniform float viewHeight;

uniform ivec2 eyeBrightnessSmooth;

uniform mat4 gbufferProjectionInverse;

uniform sampler2D texture;

float time = float(worldTime);
float tB = max(sin(time/12000*22/7),0.0);
float eBS = eyeBrightnessSmooth.y/240.0;
float sunVisibility = pow(clamp(dot(sunVec,upVec)+0.1,0.0,0.1)/0.1,2.0);

#include "lib/color/lightColor.glsl"
#include "lib/color/torchColor.glsl"

void main() {
	vec4 albedo = vec4(0.0);
	
	#ifdef Weather
	//Texture
	albedo = texture2D(texture, texcoord.xy);
	
	albedo.a *= rainStrength * length(albedo.rgb/3)*float(albedo.a > 0.1);
	albedo.rgb *= (0.2 * sqrt(ambient) * (1.0-0.8*(1.0-sunVisibility)*rainStrength) + lmcoord.x * lmcoord.x * torch_c) * WeatherOpacity;
	#endif
	
/* DRAWBUFFERS:0 */

	gl_FragData[0] = albedo;
}