float distx(float dist){
	return (far * (dist - near)) / (dist * (far - near));
}

float getDepth(float depth) {
    return 2.0 * near * far / (far + near - (2.0 * depth - 1.0) * (far - near));
}

vec4 distortShadow(vec4 shadowpos, float distortFactor) {
shadowpos.xy *= 1.0f / distortFactor;
shadowpos.z = shadowpos.z*0.2;
shadowpos = shadowpos * 0.5f + 0.5f;

return shadowpos;
}

vec4 getShadowSpace(float shadowdepth, vec2 texcoord){
	vec4 fragpos = gbufferProjectionInverse * (vec4(texcoord.xy,shadowdepth,1.0)*2.0-1.0);
	fragpos /= fragpos.w;

	vec4 wpos = gbufferModelViewInverse * fragpos;
	wpos = shadowModelView * wpos;
	wpos = shadowProjection * wpos;
	wpos /= wpos.w;
	
	float distb = sqrt(wpos.x * wpos.x + wpos.y * wpos.y);
	float distortFactor = 1.0 - shadowMapBias + distb * shadowMapBias;
	wpos = distortShadow(wpos,distortFactor);
	
	return wpos;
}

//Volumetric light from Robobo1221 (modified)
vec3 getVolumetricRays(float pixeldepth0, float pixeldepth1, vec3 color, float dither) {
	vec3 vl = vec3(0.0);
	float slength = 2.25;
	vec3 watercol = water_c*sqrt(cmult)/sqrt(water_a);

	float maxDist = 128.0;
	float minDist = 0.05;
	vec4 worldposition = vec4(0.0);
	float w = 0.0;
	
	float depth0 = getDepth(pixeldepth0);
	float depth1 = getDepth(pixeldepth1);
	
	for (minDist; minDist < maxDist; ) {
		minDist += dither*slength;
		if (depth1 < minDist || (depth0 < minDist && color == vec3(0.0))){
			break;
		}
		worldposition = getShadowSpace(distx(minDist),texcoord.st);
		if (length(worldposition.xy*2.0-1.0)<1.0){
			vec3 sample = vec3(shadow2D(shadowtex0, vec3(worldposition.rg, worldposition.b+0.00002)).z);
			vec3 colsample = vec3(0.0);
			#ifdef ShadowColor
			float testsample = shadow2D(shadowtex1, vec3(worldposition.rg, worldposition.b+0.00002)).z;
			if (testsample-sample.r > 0.9) colsample = shadow2D(shadowcolor, vec3(worldposition.rg, worldposition.b)).rgb;
			#endif
			sample = pow(max(sample,colsample),vec3(2.2));
			if (depth0 < minDist) sample *= color;
			else sample *= mix(vec3(1.0),watercol,isEyeInWater);
			vl += sample*minDist/(0.25+minDist);
		}
		else{
			vl += 1.0;
		}
		minDist += slength*(1.0-dither);
		slength *= 1.7;
		w += 1.0;
	}
	vl /= 8.0+w;
	
	return vec3(vl);
}